#include "jngen.h"
#include <cassert>
#include <iostream>
#include <random>
#include <vector>
 #include <map>
#include <deque>
#include <numeric>
#include <memory>
#include <cstring>

using namespace std;

long long inv(long long a, long long m) {
    if (a == 1)
        return 1;
    return (1 - 1ll * inv(m % a, a) * m) / a + m;
}

const long long inf = 1e18;

struct node {
    node *l = nullptr;
    node *r = nullptr;
    node *parent = nullptr;
    long long x, add = 0, max;
    int size = 0, count;
    int y;

    node(int x): x(x) {
        y = rand();
        size = 1;
        max = x;
        count = 1;
    }
};

int s(node *v) {
    if (!v) {
        return 0;
    }
    return v->size;
}

long long max(node *v) {
    if (!v) {
        return -inf;
    }
    return v->max;
}

long long add(node *v) {
    if (!v) {
        return 0;
    }
    return v->add;
}

int cmax(node *v) {
    if (!v) {
        return 0;
    }
    return v->count;
}

void upd(node *v) {
    if (!v) {
        return;
    }
    v->size = s(v->l) + s(v->r) + 1;
    v->max = max(max(v->l) + add(v->l), max(max(v->r) + add(v->r), v->x));
    if (v->x == v->max) {
        v->count = 1;
    } else {
        v->count = 0;
    }
    if (max(v->l) + add(v->l) == v->max) {
        v->count += cmax(v->l);
    }
    if (max(v->r) + add(v->r) == v->max) {
        v->count += cmax(v->r);
    }
    if (v->l) {
        v->l->parent = v;
    }
    if (v->r) {
        v->r->parent = v;
    }
    v->parent = nullptr;
}

void push(node *v) {
    if (!v) {
        return;
    }
    if (v->l) {
        v->l->add += v->add;
    }
    if (v->r) {
        v->r->add += v->add;
    }
    v->x += v->add;
    v->max += v->add;
    v->add = 0;
}

node *merge(node *l, node *r) {
    if (!l) {
        return r;
    }
    if (!r) {
        return l;
    }
    if (l->y > r->y) {
        push(l);
        l->r = merge(l->r, r);
        upd(l);
        return l;
    }
    push(r);
    r->l = merge(l, r->l);
    upd(r);
    return r;
}

std::pair<node *, node *> split(node *v, int k) {
    if (!v) {
        return { nullptr, nullptr };
    }
    push(v);
    if (s(v->l) + 1 <= k) {
        std::pair<node *, node *> q = split(v->r, k - s(v->l) - 1);
        v->r = q.first;
        upd(v);
        return {v, q.second};
    }
    std::pair<node *, node *> q = split(v->l, k);
    v->l = q.second;
    upd(v);
    return {q.first, v};
}

int get_id(node *v) {
    int id = s(v->l);
    node *ptr = v->parent;
    while (ptr) {
        if (ptr->r == v) {
            id += s(ptr->l) + 1;
        }
        ptr = ptr->parent;
        v = v->parent;
    }
    return id;
}

int c = 1;
const int MAXN = 300'000;
const int LOG = 19;
int h[MAXN];
int up[LOG][MAXN];
int parent[MAXN];
node *vin[MAXN];
node *vout[MAXN];
int v1 = 0, v2 = 0; // diam
int center = 0;
node *root = nullptr;

namespace solver {

    int go_up(int u, int k) {
        for (int i = LOG - 1; i >= 0; --i) {
            if (k >= (1 << i)) {
                k -= (1 << i);
                u = up[i][u];
            }
        }
        return u;
    }

    int lca(int u, int v) {
        if (h[u] < h[v]) {
            swap(u, v);
        }
        u = go_up(u, h[u] - h[v]);
        if (u == v) {
            return u;
        }
        for (int i = LOG - 1; i >= 0; --i) {
            if (up[i][u] != up[i][v]) {
                u = up[i][u];
                v = up[i][v];
            }
        }
        return up[0][u];
    }

    int dist(int u, int v) {
        return h[u] + h[v] - 2 * h[lca(u, v)];
    }

    int getf(int v) {
        return get_id(vin[v]);
    }

    int getl(int v) {
        return get_id(vout[v]);
    }

    void add_seg(int l, int r, int x) {
        std::pair<node *, node *> q = split(root, r + 1);
        std::pair<node *, node *> q1 = split(q.first, l);
        q1.second->add += x;
        root = merge(q1.first, merge(q1.second, q.second));
    }

    int get_count(int l, int r) {
        std::pair<node *, node *> q = split(root, r + 1);
        std::pair<node *, node *> q1 = split(q.first, l);
        int res = cmax(q1.second);
        root = merge(q1.first, merge(q1.second, q.second));
        return res;
    }

    int get_count1(int l, int r) {
        std::pair<node *, node *> q = split(root, r);
        std::pair<node *, node *> q1 = split(q.first, l);
        node *t = merge(q1.first, q.second);
        int res = cmax(t);
        std::pair<node *, node *> q2 = split(t, l);
        root = merge(q2.first, merge(q1.second, q2.second));
        return res;
    }

    long long getVal(int i, node* v = root) {
        pair<node *, node *> q = split(v, i + 1);
        pair<node *, node *> q1 = split(q.first, i);
        long long res = q1.second->x + q1.second->add;
        root = merge(q1.first, merge(q1.second, q.second));
        return res;
    }

    void move_along_edge(int to) {
        int from = center;
        int w = 1;
        if (to == parent[from]) {
            w = -1;
        }
        center = to;
        int L = getf(from);
        int R = getf(to);
        if (L > R) {
            swap(from, to);
            swap(L, R);
        }
        swap(L, R);
        R = getl(to);
        add_seg(0, 2 * c - 1, w);
        add_seg(L, R, -w * 2);
    }

    void move_center(int added) {
        int len = dist(v1, v2);
        int x = dist(v1, added);
        bool check_needed = false;
        if (len < x) {
            len = x;
            v2 = added;
            check_needed = true;
        }
        int y = dist(v2, added);
        if (len < y) {
            len = y;
            v1 = added;
            check_needed = true;
        }
        if (check_needed) {
            int dst = dist(center, added);
            if (dst > len / 2) {
                if (dst > dist(added, parent[center])) {
                    move_along_edge(parent[center]);
                } else {
                    int u = go_up(added, dst - 1);
                    move_along_edge(u);
                }
            }
        }
    }

    int ans(int v) {
        int total = cmax(root);
        int dst = dist(v, center);
        if (v == center) {
            return total;
        }
        if (h[v] - h[center] == dst) {
            // subtree
            int u = go_up(v, dst - 1);
            return get_count1(getf(u), getl(u));
        } else {
            // parent
            return get_count(getf(center), getl(center));
        }
    }

    void add_vertex(int p, int v) {
        h[v] = h[p] + 1;
        parent[v] = p;
        up[0][v] = p;
        for (int i = 1; i < LOG; ++i) {
            up[i][v] = up[i - 1][up[i - 1][v]];
        }
        int len = dist(v, center);
        vin[v] = new node(len);
        vout[v] = new node(len);
        int i = getf(p);
        std::pair<node *, node *> q = split(root, i + 1);
        root = merge(q.first, merge(merge(vin[v], vout[v]), q.second));
    }

    void init() {
        vin[0] = new node(0);
        vout[0] = new node(0);
        root = merge(vin[0], vout[0]);
    }

    void add(int v) {
        add_vertex(v, c);
        ++c;
        move_center(c - 1);
    }

    pair<int, int> answer(int v) {
        return {max(dist(v1, v), dist(v2, v)), ans(v) / 2};
    }
}

int main(int argc, char* argv[]) {
    parseArgs(argc, argv);
    registerGen(argc, argv, 1);
    int q = getOpt("q");
    bool prefix = hasOpt("prefix");
    bool v1 = hasOpt("v1");
    bool v1_c = hasOpt("v1c");
    int online = hasOpt("online");
    std::string type = getOpt("type");
    Tree t;
    int tree_size = q * 3 / 4;
    if (type == "kary") {
        int k = getOpt("k");
        t = Tree::kary(tree_size, k);
    } else if (type == "random") {
        t = Tree::random(tree_size);
    } else if (type == "binary") {
        t = Tree::binary(tree_size);
    } else if (type == "kruskal") {
        t = Tree::randomKruskal(tree_size);
    } else if (type == "star") {
        int s = tree_size / 10;
        int total = s;
        int small = 1000;
        t = Tree::caterpillar(s, s / 10);
        vector<vector<int>> g(s);
        for (auto e : t.edges()) {
            g[e.first].push_back(e.second);
            g[e.second].push_back(e.first);
        }
        vector<int> leaves;
        for (int i = 0; i < s; ++i) {
            if (g[i].size() == 1) {
                leaves.push_back(i);
            }
        }
        while (total + small < tree_size) {
            Tree t1 = Tree::star(small);
            int v = leaves[rnd.next(0, (int)leaves.size() - 1)];
            t = t.link(v, t1, 0);
            total += small;
            small = rnd.next(750, 1250);
        }
        tree_size = total;
        t = t.shuffled();
    } else if (type == "combined") {
        int s1 = tree_size / 4;
        int s2 = tree_size / 2;
        t = Tree::star(s1);
        Tree t1 = Tree::bamboo(s2);
        Tree t2 = Tree::random(tree_size - s1 - s2);
        t1 = t1.link(s2 - 1, t2, 0);
        t = t.link(rnd.next(0, s1 - 1), t1, 0);
        t = t.shuffled();
    } else if (type == "blocks") {
        int block_size = tree_size / 10;
        vector<Tree> vec(10);
        for (int i = 0; i < 3; ++i) {
            vec[i * 3] = Tree::kary(block_size, 3);
            vec[i * 3 + 1] = Tree::binary(block_size);
            vec[i * 3 + 2] = Tree::kary(block_size, 5);
        }
        vec.back() = Tree::binary(tree_size - 9 * block_size);
        t = vec[0];
        for (int i = 1; i < 10; ++i) {
            t = t.link(rnd.next(block_size * (i - 1), block_size * i - 1), vec[i], rnd.next(0, vec[i].n() - 1));
        }
        t = t.shuffled();
    } else if (type == "random_blocks") {
        int block_size = tree_size / 10;
        vector<Tree> vec(10);
        for (int i = 0; i < 3; ++i) {
            vec[i * 3] = Tree::random(block_size);
            vec[i * 3 + 1] = Tree::binary(block_size);
            vec[i * 3 + 2] = Tree::kary(block_size, 5);
        }
        vec.back() = Tree::randomKruskal(tree_size - 9 * block_size);
        t = vec[0];
        for (int i = 1; i < 10; ++i) {
            t = t.link(rnd.next(block_size * (i - 1), block_size * i - 1), vec[i], rnd.next(0, vec[i].n() - 1));
        }
        t = t.shuffled();
    }
    if (v1_c) {
        t = Tree::randomPrim(tree_size, -1000);
    }
    vector<int> dist(tree_size, tree_size);
    vector<vector<int>> g(tree_size);
    vector<pair<int, int>> edges;
    for (auto e : t.edges()) {
        g[e.first].push_back(e.second);
        g[e.second].push_back(e.first);
    }
    deque<int> que;
    for (int i = 0; i < tree_size; ++i) {
        if (g[i].size() == 1) {
            que.push_back(i);
            dist[i] = 0;
        }
    }
    while (!que.empty()) {
        int v = que.front();
        que.pop_front();
        for (int u : g[v]) {
            if (dist[u] > dist[v] + 1) {
                dist[u] = dist[v] + 1;
                que.push_back(u);
            }
        }
    }
    que.push_back(0);
    vector<int> sorted(tree_size);
    sorted[0] = 1;
    int curr = 2;
    while (!que.empty()) {
        int v = que.front();
        que.pop_front();
        for (int u : g[v]) {
            if (!sorted[u]) {
                sorted[u] = curr++;
                que.push_back(u);
            }
        }
    }
    for (auto e : t.edges()) {
        e.first = sorted[e.first];
        e.second = sorted[e.second];
        if (e.first < e.second) {
            swap(e.first, e.second);
        }
        edges.push_back(e);
    }
    sort(edges.begin(), edges.end());
    vector<int> all;
    vector<pair<int, int>> vec;
    for (int i = 0; i < tree_size; ++i) {
        vec.push_back(make_pair(dist[i], i));
    }
    sort(vec.begin(), vec.end());
    int j = 0;
    if (type == "combined") {
        int k =  q - tree_size + 1;
        for (int i = 0; i < k; ++i) {
            if (rnd.next(0, 10) == 0) {
                all.push_back(sorted[rnd.next(0, tree_size / 2 - 1) + tree_size / 4]);
            } else {
                all.push_back(sorted[rnd.next(tree_size / 4 + tree_size / 2, tree_size - 1)]);
            }
        }
    } else {
        for (int i = 0; i < tree_size && all.size() < (q - tree_size + 1) * 7 / 8; i += 1) {
            while (j < tree_size && vec[j].first == i) {
                ++j;
            }
            if (j > 0 && vec[j - 1].first == i) {
                all.push_back(sorted[vec[j - 1].second]);
            }
        }
        for (int i = all.size(); i < q - tree_size + 1; ++i) {
            all.push_back(sorted[rnd.next(0, tree_size - 1)]);
        }
    }
    sort(all.begin(), all.end());
    cout << q << ' ' << online << '\n';
    j = 0;
    if (v1) {
        fill(all.begin(), all.end(), 1);
    } else if (v1_c) {
        const int k = g[0].size();
        for (int i = 0; i < all.size(); ++i) {
            all[i] = sorted[g[0][rnd.next(0, k-1)]];
        }
        sort(all.begin(), all.end());
    }
    if (prefix) {
        ensure(online == 0, "this is an offline generator");
        for (int i = 0; i < edges.size(); ++i) {
            cout << "+ " << edges[i].second << '\n';
        }
        for (int i = 0; i < all.size() ; ++i) {
            cout << "? " << all[i] << '\n';
        }
    } else {
        j = 0;
        int x = 0;
        vector<int> possible;
        solver::init();
        int ans = 0;
        for (int i = 0; i < edges.size(); ++i) {
            int v = (edges[i].second - 1) % c + 1;
            int query = v;
            if (online) {
                if (ans != 0) {
                    query = (((query - ans) % c + c) % c) % c;
                }
            }
            cout << "+ " << query << '\n';
            solver::add(v - 1);
            x = i + 2;
            while (j < all.size() && all[j] < x) {
                possible.push_back(all[j]);
                ++j;
            }
            while (rnd.next(0, 1) && !possible.empty()) {
                int i = rnd.next(0, (int)possible.size() - 1);
                int v = (possible[i] - 1) % c + 1;
                int query = v;
                if (online) {
                    if (ans != 0) {
                        query = (((v - ans) % c + c) % c) % c;
                    }
                }
                auto[x, y] = solver::answer(v - 1);
                ans = x + y;
                cout << "? " << query << '\n';
                swap(possible[i], possible.back());
                possible.pop_back();
            }
        }
        while (j < all.size()) {
            possible.push_back(all[j]);
            ++j;
        }
        shuffle(possible.begin(), possible.end());
        for (int i : possible) {
            int v = (i - 1) % c + 1;
            int query = v;
            if (online) {
                if (ans != 0) {
                    query = (((v - ans) % c + c) % c);
                }
            }
            auto[x, y] = solver::answer(v - 1);
            ans = x + y;
            cout << "? " << query << '\n';
        }
    }
}
