#include "testlib.h"
#include <iostream>
#include <vector>
#include <map>
#include <assert.h>
#include <set>
#include <deque>
#include <algorithm>
#include <random>

using namespace std;

const int32_t MAXN = 300'000;

struct Bound {
    long long q, d, m;
};

vector<Bound> MAX{
    // 0: sample
    {MAXN, MAXN, 1},
    // 1: q <= 100
    {100, MAXN, 0},
    // 2: + before ?
    {MAXN, MAXN, 0},
    // 3: d <= 50
    {MAXN, 50, 0},
    // 4: vi = 1
    {MAXN, MAXN, 1},
    // 5: vi = 1,child of 1
    {MAXN, MAXN, 1},
    // 6: m = 0
    {MAXN, MAXN, 0},
    // 7: m = 1
    {MAXN, MAXN, 1},
};

int group;

void checkConstraints(int q, int m) {
    ensuref(group >= 0 && group <= 9, "Incorrect group: %d", group);
    ensuref(q <= MAX[group].q, "wrong q for group %d", group);
    ensuref(m <= MAX[group].m, "wrong m for group %d", group);
}

vector<int> dist(int v, vector<vector<int>> &g) {
    vector<int> all(g.size(), 0);
    deque<int> q{v};
    all[v] = 1;
    while (!q.empty()) {
        int v = q.front();
        q.pop_front();
        for (int u : g[v]) {
            if (!all[u]) {
                all[u] = all[v] + 1;
                q.push_back(u);
            }
        }
    }
    return all;
}

int diam(vector<vector<int>> &g) {
    auto x = dist(0, g);
    int v1 = max_element(x.begin(), x.end()) - x.begin();
    auto y = dist(v1, g);
    int v2 = max_element(y.begin(), y.end()) - y.begin();
    auto z = dist(v2, g);
    return *max_element(z.begin(), z.end()) - 1;
}

void checkGroup(int q, vector<vector<int>> &g, vector<int> &queries, vector<char> &str) {
    ensuref(diam(g) <= MAX[group].d, "Incorrect diam for group: %d", group);
    if (group == 4) {
        ensuref(count(queries.begin(), queries.end(), 0) == queries.size(), "Unsatisfied vi=1 for group %d", group);
    }
    if (group == 5) {
        set<int> s;
        s.insert(0);
        for (int i : g[0]) {
            s.insert(i);
        }
        for (int j : queries) {
            ensuref(s.count(j), "Unsatisfied conditition for group %d", group);
        }
    }
    if (group == 3) {
        int i = str.size();
        for (int j = 0; i < str.size(); ++j) {
            if (str[j] == '?') {
                i = j;
            }
        }
        for (int j = i + 1; j < str.size(); ++j) {
            ensuref(str[j] == '?', "Unsatisfied condition for group 3");
        }
    }
}

const long long INF = 1e18;
mt19937 rnd1;

struct node {
    node *l = nullptr;
    node *r = nullptr;
    node *parent = nullptr;
    long long x, add = 0, max;
    int size = 0, count;
    int y;

    node(int x): x(x) {
        y = rnd1();
        size = 1;
        max = x;
        count = 1;
    }
};

int s(node *v) {
    if (!v) {
        return 0;
    }
    return v->size;
}

long long max(node *v) {
    if (!v) {
        return -INF;
    }
    return v->max;
}

long long add(node *v) {
    if (!v) {
        return 0;
    }
    return v->add;
}

int cmax(node *v) {
    if (!v) {
        return 0;
    }
    return v->count;
}

void upd(node *v) {
    if (!v) {
        return;
    }
    v->size = s(v->l) + s(v->r) + 1;
    v->max = max(max(v->l) + add(v->l), max(max(v->r) + add(v->r), v->x));
    if (v->x == v->max) {
        v->count = 1;
    } else {
        v->count = 0;
    }
    if (max(v->l) + add(v->l) == v->max) {
        v->count += cmax(v->l);
    }
    if (max(v->r) + add(v->r) == v->max) {
        v->count += cmax(v->r);
    }
    if (v->l) {
        v->l->parent = v;
    }
    if (v->r) {
        v->r->parent = v;
    }
    v->parent = nullptr;
}

void push(node *v) {
    if (!v) {
        return;
    }
    if (v->l) {
        v->l->add += v->add;
    }
    if (v->r) {
        v->r->add += v->add;
    }
    v->x += v->add;
    v->max += v->add;
    v->add = 0;
}

node *merge(node *l, node *r) {
    if (!l) {
        return r;
    }
    if (!r) {
        return l;
    }
    if (l->y > r->y) {
        push(l);
        l->r = merge(l->r, r);
        upd(l);
        return l;
    }
    push(r);
    r->l = merge(l, r->l);
    upd(r);
    return r;
}

std::pair<node *, node *> split(node *v, int k) {
    if (!v) {
        return { nullptr, nullptr };
    }
    push(v);
    if (s(v->l) + 1 <= k) {
        std::pair<node *, node *> q = split(v->r, k - s(v->l) - 1);
        v->r = q.first;
        upd(v);
        return {v, q.second};
    }
    std::pair<node *, node *> q = split(v->l, k);
    v->l = q.second;
    upd(v);
    return {q.first, v};
}

int get_id(node *v) {
    int id = s(v->l);
    node *ptr = v->parent;
    while (ptr) {
        if (ptr->r == v) {
            id += s(ptr->l) + 1;
        }
        ptr = ptr->parent;
        v = v->parent;
    }
    return id;
}

int c = 1;
const int LOG = 19;
int h[MAXN];
int up[LOG][MAXN];
int parent[MAXN];
node *vin[MAXN];
node *vout[MAXN];
int v1 = 0, v2 = 0; // diam
int center = 0;
node *root = nullptr;

namespace solver {

    int go_up(int u, int k) {
        for (int i = LOG - 1; i >= 0; --i) {
            if (k >= (1 << i)) {
                k -= (1 << i);
                u = up[i][u];
            }
        }
        return u;
    }

    int lca(int u, int v) {
        if (h[u] < h[v]) {
            swap(u, v);
        }
        u = go_up(u, h[u] - h[v]);
        if (u == v) {
            return u;
        }
        for (int i = LOG - 1; i >= 0; --i) {
            if (up[i][u] != up[i][v]) {
                u = up[i][u];
                v = up[i][v];
            }
        }
        return up[0][u];
    }

    int dist(int u, int v) {
        return h[u] + h[v] - 2 * h[lca(u, v)];
    }

    int getf(int v) {
        return get_id(vin[v]);
    }

    int getl(int v) {
        return get_id(vout[v]);
    }

    void add_seg(int l, int r, int x) {
        std::pair<node *, node *> q = split(root, r + 1);
        std::pair<node *, node *> q1 = split(q.first, l);
        q1.second->add += x;
        root = merge(q1.first, merge(q1.second, q.second));
    }

    int get_count(int l, int r) {
        std::pair<node *, node *> q = split(root, r + 1);
        std::pair<node *, node *> q1 = split(q.first, l);
        int res = cmax(q1.second);
        root = merge(q1.first, merge(q1.second, q.second));
        return res;
    }

    int get_count1(int l, int r) {
        std::pair<node *, node *> q = split(root, r);
        std::pair<node *, node *> q1 = split(q.first, l);
        node *t = merge(q1.first, q.second);
        int res = cmax(t);
        std::pair<node *, node *> q2 = split(t, l);
        root = merge(q2.first, merge(q1.second, q2.second));
        return res;
    }

    long long getVal(int i, node* v = root) {
        pair<node *, node *> q = split(v, i + 1);
        pair<node *, node *> q1 = split(q.first, i);
        long long res = q1.second->x + q1.second->add;
        root = merge(q1.first, merge(q1.second, q.second));
        return res;
    }

    void move_along_edge(int to) {
        int from = center;
        int w = 1;
        if (to == parent[from]) {
            w = -1;
        }
        center = to;
        int L = getf(from);
        int R = getf(to);
        if (L > R) {
            swap(from, to);
            swap(L, R);
        }
        swap(L, R);
        R = getl(to);
        add_seg(0, 2 * c - 1, w);
        add_seg(L, R, -w * 2);
    }

    void move_center(int added) {
        int len = dist(v1, v2);
        int x = dist(v1, added);
        bool check_needed = false;
        if (len < x) {
            len = x;
            v2 = added;
            check_needed = true;
        }
        int y = dist(v2, added);
        if (len < y) {
            len = y;
            v1 = added;
            check_needed = true;
        }
        if (check_needed) {
            int dst = dist(center, added);
            if (dst > len / 2) {
                if (dst > dist(added, parent[center])) {
                    move_along_edge(parent[center]);
                } else {
                    int u = go_up(added, dst - 1);
                    move_along_edge(u);
                }
            }
        }
    }

    int ans(int v) {
        int total = cmax(root);
        int dst = dist(v, center);
        if (v == center) {
            return total;
        }
        if (h[v] - h[center] == dst) {
            // subtree
            int u = go_up(v, dst - 1);
            return get_count1(getf(u), getl(u));
        } else {
            // parent
            return get_count(getf(center), getl(center));
        }
    }

    void add_vertex(int p, int v) {
        h[v] = h[p] + 1;
        parent[v] = p;
        up[0][v] = p;
        for (int i = 1; i < LOG; ++i) {
            up[i][v] = up[i - 1][up[i - 1][v]];
        }
        int len = dist(v, center);
        vin[v] = new node(len);
        vout[v] = new node(len);
        int i = getf(p);
        std::pair<node *, node *> q = split(root, i + 1);
        root = merge(q.first, merge(merge(vin[v], vout[v]), q.second));
    }

    void init() {
        vin[0] = new node(0);
        vout[0] = new node(0);
        root = merge(vin[0], vout[0]);
    }

    void add(int v) {
        add_vertex(v, c);
        ++c;
        move_center(c - 1);
    }

    pair<int, int> answer(int v) {
        return {max(dist(v1, v), dist(v2, v)), ans(v) / 2};
    }
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    group = validator.group().empty() ? 0 : atoi(validator.group().c_str());
    int32_t q = inf.readInt(1, MAXN, "q");
    inf.readSpace();
    int32_t m = inf.readInt(0, 1, "m");
    inf.readEoln();
    vector<vector<int>> g(q + 1);
    vector<int> queries;
    vector<char> str;
    solver::init();
    int answ = 0;
    for (int i = 0; i < q; ++i) {
        std::string t;
        int v;
        t = inf.readWord("\+|\?", "t");
        ensuref(t == "+" || t == "?", "t should be in {+, ?}");
        inf.readSpace();
        v = inf.readInt(0, q, "v");
        inf.readEoln();
        str.push_back(t[0]);
        v = (answ * m + v - 1 + c) % c;
        if (t == "+") {
            g[v].push_back(c);
            g[c].push_back(v);
            solver::add(v);
        } else {
            queries.push_back(v);
            auto [x, y] = solver::answer(v);
            answ = x + y;
        }
    }
    inf.readEof();
    checkConstraints(q, m);
    checkGroup(q, g, queries, str);
}
